// Declare global variables
var AlipayJSBridge;

(function(){
  var nativeLog = self.__nativeLog__;
  var nativeCreateTimer = self.__nativeCreateTimer__;
  var nativeDeleteTimer = self.__nativeDeleteTimer__;
  var nativeFlushQueue = self.__nativeFlushQueue__;

  function nop(){}
  function returnEmptyArray(){ return []; }

  function log() {
    try {
      var msg = '';
      for (var i = 0; i < arguments.length; i++) {
        var value = arguments[i];
        if(i > 0) msg += ' ';
        if (value instanceof Error) {
          msg += value.message + "\n" + value.stack;
        }
        else {
          msg += (typeof value == 'object') ? JSON.stringify(value) : value;
        }
      }
      nativeLog(msg);
    } catch(err) {}
  }

  function createTimer(args, repeat) {
    var fn = args[0];
    if (typeof fn === 'string') {
      fn = new Function(fn);
    }
    else if (typeof fn !== 'function') {
      return -1;
    }

    var delay = args[1] | 0;
    var aArgs = Array.prototype.slice.call(args, 2);
    return nativeCreateTimer(function(){
      try {
        fn.apply(self, aArgs);
      } catch (err) {
        console.error("###\n" + err.stack + "\n###");
        if (typeof self.onerror === 'function') {
          self.onerror(err);
        }
      }
    }, delay < 0 ? 0 : delay, repeat);
  }

  if (nativeLog) self.__nativeLog__ = null;
  self.__nativeCreateTimer__ = null;
  self.__nativeDeleteTimer__ = null;
  self.__nativeFlushQueue__ = null;

  self.setTimeout = function() {
    return createTimer(arguments, false);
  };

  self.setInterval = function() {
    return createTimer(arguments, true);
  };

  self.clearTimeout = self.clearInterval = function(id) {
    nativeDeleteTimer(id | 0);
  };

  self.close = nop;
  self.postMessage = nop;
  self.performance = {
    clearMarks: nop,
    clearMeasures: nop,
    clearResourceTimings: nop,
    getEntries: returnEmptyArray,
    getEntriesByName: returnEmptyArray,
    getEntriesByType: returnEmptyArray,
    setResourceTimingBufferSize: nop,
    mark: nop,
    measure: nop,
    now: function(){ return Date.now() },
    toJSON: function(){ return {}; }
  };

  self.onerror = function(err) {
    try {
      log("###############################################");
      log(err);
      log("###############################################");
    } catch(e) {}
  };

  if (nativeLog) {
    self.console = {
      log: log,
      info: log,
      debug: log,
      warn: log,
      error: log,
      exception: log,
      clear: nop,
      count: nop,
      countReset: nop,
      dir: nop,
      dirxml: nop,
      group: nop,
      groupCollapsed: nop,
      groupEnd: nop,
      profile: nop,
      profileEnd: nop,
      table: nop,
      time: nop,
      timeEnd: nop,
      timeLog: nop,
      timeStamp: nop,
      trace: nop,
      assert: function() {
        if (arguments[0]) return;
        console.warn("Assertion failed:", Array.prototype.slice.call(arguments).slice(1));
      }
    };
  }


  var handlers = {};
  var windowHandlers = {};
  var sendMessageQueue = [];

  function Event(name) {
    this.name = name;
  }

  Event.prototype = {
    preventDefault: function () {
      this._preventDefault = true;
    },
    initEvent: function(name) {
      this.name = name;
    },
    isDefaultPrevented: function () {
      return !!this._preventDefault;
    }
  };
  function add(eventName, fn) {
    handlers[eventName] || (handlers[eventName] = []);

    var handler = {};
    handler.fn = fn;

    handlers[eventName].push(handler);
  }

  function windowAddEvent(eventName, fn) {
    windowHandlers[eventName] || (windowHandlers[eventName] = []);

    var handler = {};
    handler.fn = fn;

    windowHandlers[eventName].push(handler);
  }

  function remove(eventName, fn) {
    var eventHandlers = handlers[eventName];
    if (eventHandlers) {
      for (var i = 0; i < eventHandlers.length; i++) {
        if (fn === eventHandlers[i].fn) {
          eventHandlers.splice(i, 1);
          if (eventHandlers.length === 0) {
            delete handlers[eventName];
          }
          break;
        }
      }
    }
  }

  function removeWindowEvent(eventName, fn) {
    if (windowHandlers[eventName]) {
      for (var i = 0; i < windowHandlers[eventName].length; i++) {
        if (fn === windowHandlers[eventName][i].fn) {
          windowHandlers[eventName].splice(i, 1);
          if (windowHandlers[eventName].length === 0) {
            delete windowHandlers[eventName];
          }
          break;
        }
      }
    }
  }

  function createEvent(eventName) {
    return new Event(eventName);
  }

  function dispatchEvent(eventName, args) {
    var evt = createEvent(eventName);
    var isDefaultPrevented = false;
    var eventHandlers = handlers[eventName];
    if (eventHandlers) {
      for (var i = 0; i < eventHandlers.length; i++) {
        evt.data = args;
        try {
          eventHandlers[i].fn(evt);
          if (evt.isDefaultPrevented()) {
            isDefaultPrevented = true;
          }
        } catch (err) {
          console.error("###dispatchEvent###\n" + err.stack + "\n###");
          if (typeof self.onerror === 'function') {
            self.onerror(err);
          }
        }
      }
    }

    return !isDefaultPrevented;
  }

  function trigger(eventName, args) {
        var isDefaultPrevented = !dispatchEvent(eventName, args);
        return isDefaultPrevented;
  }

  var https_appx = "https://appx/";

  var document = {
    addEventListener: add,
    removeEventListener: remove,
    trigger: trigger,
    sendMessageQueue: sendMessageQueue,
    createEvent: createEvent,
    dispatchEvent: dispatchEvent,
    location: {
      _href: https_appx,
      search: "",
      hash: "",
      host: "appx",
      hostname: "appx",
      origin: https_appx,
      pathname: "/",
      port: "",
      protocol: "https:",
      reload: function(){},
      replace: function(){},
      assign: function(){}
    },

    title: '',
    origin: https_appx
  };
  self.addEventListener = windowAddEvent;
  self.removeEventListener = removeWindowEvent;

  var msgPortMap = {};

  function trasMsgPort(viewId, pageId, msgPort) {
    msgPort.postMessage = function (data) {
      var _data = {
        data: data,
        type: 'messagePort',
        msgPortId: msgPort.id,
        viewId: viewId,
        pageId: pageId
      };
      AlipayJSBridge.call("postMessage", _data);
    };
    if(!msgPortMap['m_'+viewId]){
      msgPortMap['m_'+viewId] = {};
    }
    if(!msgPortMap['m_' + viewId]['p_' + pageId]){
      msgPortMap['m_' + viewId]['p_' + pageId] = {};
    }
    msgPortMap['m_'+viewId]['p_' + pageId][msgPort.id] = msgPort;
    return msgPort;
  }

  document.addEventListener('push', function (event) {
      var data = event.data;
      var pushHandlers = windowHandlers['push'];
      if (__appxStartupParams && __appxStartupParams.debug === "framework") {
        console.log("onMessage push", event);
      }
      if (pushHandlers) {
        for (var i = 0; i < pushHandlers.length; i++) {
          event.data = data;
          try {
            pushHandlers[i].fn(event);
          } catch (err) {
            console.error("###push###\n" + err.stack + "\n###");
            if (typeof self.onerror === 'function') {
              self.onerror(err);
            }
          }
        }
      }
  });

  document.addEventListener('message', function (event) {
    if (event.data && event.data.type != 'messagePort' && !event.data.beforeunload) {
      var data = event.data.data;
      var eventPorts = event.data.eventPorts;
      var viewId = event.data.viewId;
      var pageId = event.data.pageId || 0;
      if (eventPorts && eventPorts[0]) {
        eventPorts[0] = trasMsgPort(viewId, pageId, eventPorts[0]);
      }
      if (__appxStartupParams && __appxStartupParams.debug === "framework") {
        console.log("onMessage eventPorts");
      }
      var messageHandlers = windowHandlers['message'];
      if (messageHandlers) {
        for (var i = 0; i < messageHandlers.length; i++) {
          event.data = data;
          event.ports = eventPorts;
          try {
            messageHandlers[i].fn(event);
          } catch (err) {
            console.error("###message###\n" + err.stack + "\n###");
            if (typeof self.onerror === 'function') {
              self.onerror(err);
            }
          }
        }
      }
    } else if (event.data && event.data.type == 'messagePort' && event.data.msgPortId) {
      var viewId = event.data.viewId;
      var pageId = event.data.pageId || 0;
      if(!msgPortMap['m_' + viewId] || !msgPortMap['m_' + viewId]['p_' + pageId]){
        console.error('unknown view', event, msgPortMap);
        return;
      }
      var msgPort = msgPortMap['m_' + viewId]['p_' + pageId][event.data.msgPortId];
      if (msgPort && msgPort.onmessage) {
        msgPort.onmessage({
          data: event.data.data
        });
      }
    }  else {
      console.log('unknown event', event);
    }
  });

  Object.defineProperty(document.location, "href", {
    get: function () {
      return document.location._href;
    },

    set: function (newValue) {
      if (typeof newValue !== 'string') return;

      newValue = newValue.trim();
      if(AlipayJSBridge._trimLocationHref){
        var rs = AlipayJSBridge._trimLocationHref(newValue);
        if(rs){
          for(var key in rs){
            if(key != "href" && typeof rs[key] !== "function"){
                document.location[key] = rs[key];
            }
          }
        }
      }
      document.location._href = newValue;
    }
  });
    /***************Param Type*****************/
    var chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";

    function InvalidCharacterError(message) {
      this.message = message;
    }
    InvalidCharacterError.prototype = new Error;
    InvalidCharacterError.prototype.name = 'InvalidCharacterError';

    // encoder
    // [https://gist.github.com/999166] by [https://github.com/nignag]
    self.btoa = function(input) {
      var str = String(input);
      for (
          // Initialize result and counter.
          var block, charCode, idx = 0, map = chars, output = '';
          // If the next str index does not exist:
          //   change the mapping table to "="
          //   check if d has no fractional digits
          str.charAt(idx | 0) || (map = '=', idx % 1);
          // "8 - idx % 1 * 8" generates the sequence 2, 4, 6, 8.
          output += map.charAt(63 & block >> 8 - idx % 1 * 8)) {
        charCode = str.charCodeAt(idx += 3 / 4);
        if (charCode > 0xFF) {
          throw new InvalidCharacterError(
              '\'btoa\' failed: The string to be encoded contains characters ' +
              'outside of the Latin1 range.');
        }
        block = block << 8 | charCode;
      }
      return output;
    };

    // decoder
    // [https://gist.github.com/1020396] by [https://github.com/atk]
    self.atob = function(input) {
      var str = String(input).replace(/=+$/, '');
      if (str.length % 4 == 1) {
        throw new InvalidCharacterError(
            '\'atob\' failed: The string to be decoded is not ' +
            'correctly encoded.');
      }
      for (
          // Initialize result and counters.
          var bc = 0, bs, buffer, idx = 0, output = '';
          // Get next character.
          buffer = str.charAt(idx++);
          // Character found in table? initialize bit storage and add its
          // ascii value;
          ~buffer && (bs = bc % 4 ? bs * 64 + buffer : buffer,
              // And if not first of each 4 characters,
              // convert the first 8 bits to one ascii character.
              bc++ % 4) ? output += String.fromCharCode(
                    255 & bs >> (-2 * bc & 6)) : 0) {
        // Try to find character in table (0-63, not found => -1).
        buffer = chars.indexOf(buffer);
      }
      return output;
    };

  var callbackPoll = {};
  var nextCallbackId = 1;

  AlipayJSBridge = {
    call: function (func, param, callback) {

      if ('string' !== typeof func) {
        return;
      }

      if ('function' === typeof param) {
        callback = param;
        param = null;
      } else if (typeof param !== 'object') {
        param = null;
      }

      var callbackId = func + '##' + (nextCallbackId++);
      if ('function' === typeof callback) {
        callbackPoll[callbackId] = callback;
      }

      if (param && param.callbackId) {
        var msg = {
          responseId: param.callbackId,
          responseData: param
        };
        delete param.callbackId;
      } else {
        var msg = {
          callbackId: callbackId,
          handlerName: func,
          data: param || {}
        };
      }
      var viewId = (param && ('viewId' in param)) ? (param.viewId | 0) : -1;
      nativeFlushQueue(func, viewId, msg);
    },

    _invokeJS: function (invokeMsg, pageId, viewId) {
      var resp = typeof invokeMsg === "string" ? JSON.parse(invokeMsg) : invokeMsg;
      if (resp["__FastPath__"] === 1) { // begin of fastpath
        delete resp["__FastPath__"];

        if (resp.msgType === "call") {
          var func = resp.func === "postMessage" ? "message" : resp.func;
          var data = resp.param || {};
          if (resp.clientId) {
            data.callbackId = resp.clientId;
          }
          if (pageId) {
            data.pageId = pageId;
          }
          if (viewId) {
            data.viewId = viewId;
          }
          return document.trigger(func, data);
        }
        else {
          var func = callbackPoll[resp.clientId];
          if (!(typeof resp.keepCallback == 'boolean' && resp.keepCallback)) {
            delete callbackPoll[resp.clientId];
          }

          if ('function' === typeof func) {
            func(resp.param);
          }
        }
        return;
      } // end of fastpath

      if (resp.responseId) {
        var func = callbackPoll[resp.responseId];
        if (!(typeof resp.keepCallback == 'boolean' && resp.keepCallback)) {
          delete callbackPoll[resp.responseId];
        }

        if ('function' === typeof func) {
          func(resp.responseData);
        }
      } else if (resp.handlerName) {
        if (resp.data && resp.data.ariver_message) {
            var tmpData = JSON.parse(resp.data.ariver_message);
            var pageId = resp.data.pageId;
            if (pageId) {
                tmpData.pageId = pageId;
            }
            var viewId = resp.data.viewId;
            if (viewId) {
                tmpData.viewId = viewId;
            }
            resp.data = tmpData;
        }
        if (resp.callbackId) {
          resp.data = resp.data || {};
          resp.data.callbackId = resp.callbackId;
        }
        var isDefaultPrevented = document.trigger(resp.handlerName, resp.data);
        if (__appxStartupParams && __appxStartupParams.debug === "framework") {
          console.log('dispatchEvent ' + resp.handlerName + ' isDefaultPrevented ' + isDefaultPrevented);
        }
        return isDefaultPrevented;
      }
    },

  };

  self.history = {
    length: 1
  };

  self.origin = document.origin;
  self.location = document.location;
  self.document = document;
  self.window = self;

})();
